/**************************************************************

File Name:	FIR_BC.asm
		
Date Modified:	05/22/00		GJO
				05/25/00		GJO	Code verified on rev 0.1 Silicon
				09/19/04		JDT Ported to VDSP++ 3.5. replace .SEGMENT syntax

Description:
			ADSP-21364 multichannel SIMD subroutine that implements N-tap 
			finite impulse response filter (FIR) for real data.  The input and output buffers are
		 	interleaved.  The single array of filter coeficients is 
			broadcasted to both the explicit and implicit registers.
			
Calling Parameters:
			r1 = TAPS-3
		    r2 = SAMPLES
			r5 = TAPS
			b0 = pointer to delay line array
		    b1 = pointer to input array
			b9 = pointer to coeficient array 
			b10 = pointer to output array
		    m0, m8 = 2
			m1 = 1 for dline initialization
			m9 = 1 for coeficient broadcasting
		    l0 = 2*TAPS
			l9 = TAPS
		    

Assumptions:
		 	Single array of coeficients is applied to each sperate input array.
			Delay line and Input buffers stored in Block 1.
			Instructions, Output, and Coefficients stored in Block 0.
			D = delay line storage element
			C = filter coefficient
			P = product        
			S = accumulated sum
			
			Block 0:
			Coeficient buffer ={ C[0],C[1],C[2],... }
			Output buffer ={ A'[0],B'[0],A'[1],B'[1],... }

			Block 1:
			Input buffer ={ A[0],B[0],A[1],B[1],... }

Return Values:
		    b10 = output array y(n)     		fir
		    b0 = delay line array zero     	fir_init

Registers Affected:
		    r0, r4, r8, r12, i0, i1, i9, i10      

Cycle Count:
		    5 + SAMPLES(6 + TAPS) + 7 cached        fir
		    4 + TAPS cycles       					fir_init
			
Memory Usage:
			Instructions Words (48-bits):
				15(fir) + 3(fir_init) instruction words
			
			Data Words (16, 32, 40, or 64-bits):
				TAPS = Number of taps locations for coefficients (32-bits)
				2*TAPS = Number of taps locations for the delay line (32-bits)
				2*SAMPLES = Number of samples locations for the input (32-bits)
				2*SAMPLES = Number of samples locations for the output (32-bits)
				

Notes: 
			Circular Buffering Issue:
			Because SIMD or Long word access transfer two 32-bit words, programs
			must be careful when using these accesses in circular buffering.
			It is important that SIMD or Long word accesses do not cross a circular
		 	buffer boundary.  If a SIMD	mode access occurs using a circular buffer
			index register that points to the last location in the circular buffer
		 	(end of buffer), the resulting access transfers the last location in
		 	the circular buffer and the first location outside the buffer (end of 
			buffer + 1).

****************************************************************************/

#include "def21364.h"						/* Symbol Definition File */

.GLOBAL fir, fir_init;

.SECTION/PM seg_pmco;
fir: 	bit set mode1 PEYEN | CBUFEN | BDCST9;
		lcntr=r2, do filtering until lce; 		/* do loop r2=SAMPLES times */
					f0=dm(i1,m0);                /* load sample of data to f0 */    
								dm(i0,m0)=f0;                /* store input sample (f0) in delay line */ 
                            	f0=dm(i0,m0), f4=pm(i9,m9);  /* get D[0], get C[0] */ 
        f8=f0*f4,           	f0=dm(i0,m0), f4=pm(i9,m9);  /* P[0]=D[0]*C[0], get D[1], get C[1] */
        f12=f0*f4,          	f0=dm(i0,m0), f4=pm(i9,m9);  /* P[1]=D[1]*C[1], get D[2], get C[2] */
        lcntr=r1, do macs until lce;           	/* P[n]=D[n]*C[n], S[n-1]=S[n-2]+P[n-1], get D[n+1] & C[n+1] */
macs:   f12=f0*f4, 	f8=f8+f12, 	f0=dm(i0,m0), f4=pm(i9,m9);  /* P[N-1]=D[N-1]*C[N-1], S[N-2]=S[N-3]+P[N-2] */
        f12=f0*f4, 	f8=f8+f12;   			   	/* S[N-1]=S[N-2]+P[N-1] */                            
                   	f0=f8+f12;   			  	/* output (S[N-1]) in f0 */
filtering:   pm(i10,m8)=f0;						/* store filtered data */
		rts(db);                                           
		bit clr mode1 PEYEN | CBUFEN;
		nop;
		
/* Initialization for FIR delay line */
fir_init:
    lcntr=r5, do zero until lce;	/* initialize delay line to zero */
zero:   dm(i0,m1)=0;
    rts;
	nop;
	nop;


